@@ -22,7 +22,7 @@ from chalk.features import is_features_cls
 from chalk.features._class_property import classproperty, classproperty_support
 from chalk.features._encoding.pyarrow import pyarrow_to_primitive
 from chalk.features.dataframe._impl import DataFrameMeta
-from chalk.features.feature_field import CacheStrategy, Feature, VersionInfo
+from chalk.features.feature_field import Feature, VersionInfo
 from chalk.features.feature_set import ClassSource, Features, CURRENT_FEATURE_REGISTRY
 from chalk.features.feature_time import feature_time
 from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
@@ -37,6 +37,15 @@ from chalk.utils.collections import ensure_tuple
 from chalk.utils.duration import Duration, parse_chalk_duration, parse_chalk_duration_s
 from chalk.utils.metaprogramming import MISSING, set_new_attribute
 from chalk.utils.string import to_snake_case
+from chalk.stores.online_store_config import OnlineStoreConfig
+
+from chalk.features.feature_cache_strategy import (
+    CacheStrategy,
+    CacheNullsType,
+    CacheDefaultsType,
+    get_cache_settings_from_strategy,
+    get_cache_strategy_from_cache_settings,
+)
 
 T = TypeVar("T")
 
@@ -54,6 +63,8 @@ def features(
     max_staleness: Optional[Duration] = None,
     name: Optional[str] = None,
     singleton: bool = False,
+    cache_nulls: CacheNullsType = True,
+    cache_defaults: CacheDefaultsType = True,
 ) -> Callable[[Type[T]], Type[T]]: ...
 
 
@@ -70,6 +81,9 @@ def features(
     max_staleness: Optional[Duration] = None,
     name: Optional[str] = None,
     singleton: bool = False,
+    online_store_config: Optional[OnlineStoreConfig] = None,
+    cache_nulls: CacheNullsType = True,
+    cache_defaults: CacheDefaultsType = True,
 ) -> Union[Callable[[Type[T]], Type[T]], Type[T]]:
     """Chalk lets you spell out your features directly in Python.
 
@@ -100,7 +114,33 @@ def features(
         value needs to have been computed to be returned without re-running a resolver.
         Assigning a `max_staleness` to the feature class assigns it to all features on the
         class which do not explicitly specify a `max_staleness` value of their own.
+    cache_nulls
+        When `True` (default), Chalk will cache all values, including nulls.
+        When `False`, Chalk will not update the null entry in the cache.
+        When `"evict_nulls"`, Chalk will evict the entry that would have been
+        null from the cache, if it exists.
 
+        Concretely, suppose the current state of a database is `{a: 1, b: 2}`,
+        and you write a row `{a: 2, b: None}`. Here is the expected result in the db:
+            - `{a: 2, b: None}` when `cache_nulls=True` (default)
+            - `{a: 2, b: 2}` when `cache_nulls=False`
+            - `{a: 2}` when `cache_nulls="evict_nulls"`
+    cache_defaults
+        When `True` (default), Chalk will cache all values, including default values.
+        When `False`, Chalk will not update the default entry in the cache.
+        When `"evict_defaults"`, Chalk will evict the entry that would have been
+        a default value from the cache, if it exists.
+
+        Concretely, suppose the current state of a database is `{a: 1, b: 2}`,
+        and you write a row `{a: 2, b: "default"}`, and the default value for feature b is `"default"`.
+        Here is the expected result in the db:
+            - `{a: 2, b: "default"}` when `cache_defaults=True`
+            - `{a: 2, b: 2}` when `cache_defaults=False`
+            - `{a: 2}` when `cache_defaults="evict_defaults"`
+
+        The `cache_nulls` and `cache_defaults` options can be used together on the same feature with the
+        following exceptions: if `cache_nulls=False`, then `cache_defaults` cannot be `"evict_defaults"`, and if
+        `cache_nulls="evict_defaults"`, then `cache_defaults` cannot be `False`.
     Other Parameters
     ----------------
     cls
@@ -180,6 +220,12 @@ def features(
                     raise_error=ValueError,
                     code="13",
                 )
+
+        cache_strategy = get_cache_strategy_from_cache_settings(
+            cache_nulls=cache_nulls,
+            cache_defaults=cache_defaults
+        )
+
         registry = CURRENT_FEATURE_REGISTRY.get()
         registry_features = registry.get_feature_sets()
         previous_features_class = registry_features.get(namespace, None)
@@ -225,8 +271,10 @@ def features(
             tags=ensure_tuple(tags),
             etl_offline_to_online=etl_offline_to_online,
             max_staleness=max_staleness,
+            cache_strategy=cache_strategy,
             namespace=namespace,
             singleton=singleton,
+            online_store_config=online_store_config,
         )
         assert is_features_cls(updated_class)
 
@@ -361,6 +409,7 @@ def _get_field(
     class_max_staleness: timedelta,
     namespace: str,
     is_singleton: bool,
+    class_cache_strategy: CacheStrategy = CacheStrategy.ALL,
 ) -> Feature:
     # Return a Field object for this field name and type.  ClassVars and
     # InitVars are also returned, but marked as such (see f._field_type).
@@ -460,6 +509,7 @@ def _get_field(
         class_tags=class_tags,
         class_etl_offline_to_online=class_etl_offline_to_online,
         class_max_staleness=class_max_staleness,
+        class_cache_strategy=class_cache_strategy,
         error_builder=error_builder,
     )
     return f
@@ -473,6 +523,7 @@ def _process_field(
     class_etl_offline_to_online: bool,
     class_max_staleness: timedelta,
     error_builder: FeatureClassErrorBuilder,
+    class_cache_strategy: CacheStrategy = CacheStrategy.ALL,
 ) -> Feature:
     comment_for_feature = comments.get(f.attribute_name)
     comment_based_description = None
@@ -533,6 +584,14 @@ def _process_field(
     if not hasattr(f, "max_staleness"):
         f.max_staleness = class_max_staleness
 
+    f_cache_nulls, f_cache_defaults = get_cache_settings_from_strategy(f.cache_strategy)
+    class_cache_nulls, class_cache_defaults =  get_cache_settings_from_strategy(class_cache_strategy)
+
+    f.cache_strategy = get_cache_strategy_from_cache_settings(
+        cache_nulls= f_cache_nulls if f_cache_nulls is not None else class_cache_nulls,
+        cache_defaults= f_cache_defaults if f_cache_defaults is not None else class_cache_defaults,
+    )
+
     # Using the private variable because the etl_offline_to_online is a read-only property
     if not hasattr(f, "etl_offline_to_online"):
         f.etl_offline_to_online = class_etl_offline_to_online
@@ -731,6 +790,8 @@ def _process_class(
     max_staleness: timedelta,
     namespace: str,
     singleton: bool,
+    online_store_config: Optional[OnlineStoreConfig],
+    cache_strategy: CacheStrategy = CacheStrategy.ALL,
 ) -> Type[T]:
     raw_cls_annotations = cls.__dict__.get("__annotations__", {})
 
@@ -741,6 +802,10 @@ def _process_class(
     group_by_materialized_windows: list[Feature] = []
     expression_windows: List[Feature] = []
     cls_annotations: Dict[str, Any] = {}
+
+    if online_store_config is not None:
+        online_store_config.feature_set_namespaces.add(namespace)
+
     for name, annotation in raw_cls_annotations.items():
         if name in ("features", "namespace", "items", "is_near"):
             error_builder.add_diagnostic(
@@ -1017,6 +1082,9 @@ def _process_class(
     del cls_annotations  # unused; set cls.__annotations__ directly
 
     set_new_attribute(cls=cls, name="__chalk_is_singleton__", value=singleton)
+    set_new_attribute(
+        cls=cls, name="__chalk_online_store_config__", value=(online_store_config and online_store_config.id)
+    )
     set_new_attribute(cls=cls, name="__chalk_error_builder__", value=error_builder)
     set_new_attribute(cls=cls, name="__chalk_source_info__", value=source_info)
     set_new_attribute(cls=cls, name="__chalk_materialized_windows__", value=materialized_windows)
@@ -1026,6 +1094,14 @@ def _process_class(
     cls_fields: List[Feature] = []
 
     if singleton:
+        if online_store_config is not None:
+            error_builder.add_diagnostic(
+                message="Singleton feature sets cannot be stored in an online store.",
+                label="invalid online store",
+                range=error_builder.decorator_kwarg_value_range("online_store_config"),
+                raise_error=ValueError,
+                code="74",
+            )
         f = Feature(
             primary=True,
             attribute_name="__chalk_singleton_id__",
@@ -1035,6 +1111,7 @@ def _process_class(
             typ=int,
             pyarrow_dtype=pa.uint8(),
             max_staleness=None,
+            cache_strategy=CacheStrategy.ALL,
             etl_offline_to_online=False,
             is_autogenerated=True,
             no_display=True,
@@ -1106,6 +1183,7 @@ def _process_class(
         ts_feature.features_cls = cls
         ts_feature.is_autogenerated = True
         cls.__annotations__[GENERATED_OBSERVED_AT_NAME] = datetime
+
         _process_field(
             f=ts_feature,
             error_builder=error_builder,
@@ -1114,6 +1192,7 @@ def _process_class(
             class_tags=tuple(cls.__chalk_tags__),
             class_etl_offline_to_online=cls.__chalk_etl_offline_to_online__,
             class_max_staleness=cls.__chalk_max_staleness__,
+            class_cache_strategy=cls.__chalk_cache_strategy__,
         )
 
         return FeatureWrapper(ts_feature)
@@ -1146,6 +1225,7 @@ def _process_class(
     set_new_attribute(cls=cls, name="__chalk_owner__", value=owner)
     set_new_attribute(cls=cls, name="__chalk_tags__", value=list(tags))
     set_new_attribute(cls=cls, name="__chalk_max_staleness__", value=max_staleness)
+    set_new_attribute(cls=cls, name="__chalk_cache_strategy__", value=cache_strategy)
     set_new_attribute(cls=cls, name="__is_features__", value=True)
     set_new_attribute(cls=cls, name="__len__", value=_len_fn)
     set_new_attribute(cls=cls, name="__getattribute__", value=_getattribute_fn)
@@ -1155,6 +1235,8 @@ def _process_class(
         name="__chalk_etl_offline_to_online__",
         value=etl_offline_to_online,
     )
+
+
     # Moving this line lower causes all kinds of problems.
     cls = classproperty_support(cls)
 
@@ -1176,6 +1258,7 @@ def _process_class(
             class_tags=tags,
             class_etl_offline_to_online=etl_offline_to_online,
             class_max_staleness=max_staleness,
+            class_cache_strategy=cache_strategy,
             namespace=namespace,
             is_singleton=singleton,
         )
@@ -1230,6 +1313,7 @@ def _process_class(
                     class_tags=tags,
                     class_etl_offline_to_online=etl_offline_to_online,
                     class_max_staleness=max_staleness,
+                    class_cache_strategy=cache_strategy,
                     error_builder=error_builder,
                 )
                 f.version.reference[i] = f_i
@@ -1436,6 +1520,7 @@ def _class_setattr(
         class_tags=tuple(cls.__chalk_tags__),
         class_etl_offline_to_online=cls.__chalk_etl_offline_to_online__,
         class_max_staleness=cls.__chalk_max_staleness__,
+        class_cache_strategy=cls.__chalk_cache_strategy__,
     )
     if existing_feature is not None:
         cls.features.remove(existing_feature)
