@@ -1,24 +1,39 @@
 from typing import Literal
 
+from typing_extensions import TypeGuard
+
 Operation = Literal[
-    "and", "or", "not", "==", "!=", "<", ">", "<=", ">", ">=", "in", "not in", "is_near_l2", "is_near_ip", "is_near_cos"
+    "!=",
+    "<",
+    "<=",
+    "==",
+    ">",
+    ">=",
+    "and",
+    "in",
+    "is_near_cos",
+    "is_near_ip",
+    "is_near_l2",
+    "not in",
+    "not",
+    "or",
 ]
 
 
-def is_valid_operation(operation: str) -> bool:
+def is_valid_operation(operation: str) -> TypeGuard[Operation]:
     return operation in (
-        "and",
-        "or",
-        "not",
-        "==",
         "!=",
-        "in",
-        "not in",
-        ">=",
-        "<=",
-        ">",
         "<",
-        "is_near_l2",
-        "is_near_ip",
+        "<=",
+        "==",
+        ">",
+        ">=",
+        "and",
+        "in",
         "is_near_cos",
+        "is_near_ip",
+        "is_near_l2",
+        "not in",
+        "not",
+        "or",
     )
